import json
import pathlib
import time
from typing import Iterable
import uuid
from pymemri.data.schema import Account, Message
from whatsapp.schema import MessageChannel

import datetime
import logging
import fire

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)
SERVICE = "sample-whatsapp"


def make_message(content):
    # This is to make sure that there is slightly different timestamp for each message
    # A better way would be to preassign a timestamp for each message
    time.sleep(0.001)

    return Message(service=SERVICE, content=content, dateSent=datetime.datetime.now())


def create_chat(
    *,
    messages: Iterable[str],
    user_names: Iterable[str] = (uuid.uuid4().hex, uuid.uuid4().hex),
    channel_name: str = uuid.uuid4().hex,
):
    """
    Create a sample database with two users and a channel.
    If no arguments are given, a random user and channel name will be generated.

    Args:
        user_names : list of user names that will be rotated through for each message
        channel_name : channel name to post messages into
    """
    logger.info(f"Creating sample database with {user_names} and {channel_name}")
    users = [Account(service=SERVICE, displayName=name) for name in user_names]
    channel = MessageChannel(service=SERVICE, name=channel_name)

    items = [*users, channel]
    items.extend((make_message(msg) for msg in messages))

    json_items = [item.to_json() for item in items]
    return json_items

def write_json(file, output_file):
    with open(file, "r") as f:
        data = json.load(f)
        chat_items = create_chat(
            messages=data["messages"],
            user_names=data["user_names"],
            channel_name=data["channel_name"],
        )
        with open(output_file, "w") as f:
            json.dump(chat_items, f, indent=2)

def main(input_folder: str = "data", output_folder: str = "output"):
    input_folder = pathlib.Path(input_folder)
    output_folder = pathlib.Path(output_folder)

    all_items = {}
    for i, file in enumerate(sorted(input_folder.glob("*.json"))):
        with open(file, "r") as f:
            data = json.load(f)
            chat_items = create_chat(
                messages=data["messages"],
                user_names=data["user_names"],
                channel_name=data["channel_name"],
            )
            all_items[i] = chat_items
    output_path = output_folder / "output.json"
    output_path.parent.mkdir(parents=True, exist_ok=True)
    with open(output_path, "w") as f:
        json.dump(all_items, f, indent=2)



if __name__ == "__main__":
    # Make Python Fire not use a pager when it prints a help text
    fire.core.Display = lambda lines, out: print(*lines, file=out)
    fire.Fire(main)
